/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.util.data;

import java.io.Serializable;

/** Pair of objects. */
public class Triple<A,B,C> implements Comparable<Triple<A,B,C>>, Serializable {
	/** Serial ID. */
	private static final long serialVersionUID = 5130820694990793690L;
	/** First element. */
	public A first;
	/** Second element. */
	public B second;
	/** Third element. */
	public C third;
	
	/** Constructor. */
	public Triple() {
	}

	/** Constructor. */
	public Triple(A first, B second, C third) {
		this.first = first;
		this.second = second;
		this.third = third;
	}
	
	/** Lexicographic comparison. */
	@SuppressWarnings("unchecked")
	public int compareTo(Triple<A, B, C> arg0) {
		int cmp = ((Comparable<A>)first).compareTo(arg0.first);
		if( cmp != 0)
			return cmp;
		cmp = ((Comparable<B>)second).compareTo(arg0.second);
		if( cmp != 0)
			return cmp;
		return ((Comparable<C>)third).compareTo(arg0.third);
	}

	@SuppressWarnings("unchecked")
	public boolean equals(Object o) {
		Triple<A, B, C> arg0 = (Triple<A, B, C>)o;
		return first.equals(arg0.first) && second.equals(arg0.second) && third.equals(arg0.third);
	}
	
	public String toString() {
		return "("+first+", "+second+", "+third+")";
	}
	
	public int hashCode() {
		return first.hashCode() + second.hashCode() + third.hashCode();
	}
}
